@@ -1,7 +1,7 @@
 #!python
 ##############################################################################
 #
-# Copyright (c) 2001, 2002 Zope Foundation and Contributors.
+# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -16,18 +16,18 @@
 
 Usage: python zdctl.py [-C URL] [-S schema.xml] [-h] [-p PROGRAM]
        [zdrun-options] [action [arguments]]
-
+ 
 Options:
--b/--backoff-limit SECONDS -- set backoff limit to SECONDS (default 10)
 -C/--configure URL -- configuration file or URL
+-S/--schema XML Schema -- XML schema for configuration file
+-h/--help -- print usage message and exit
+-b/--backoff-limit SECONDS -- set backoff limit to SECONDS (default 10)
 -d/--daemon -- run as a proper daemon; fork a subprocess, close files etc.
 -f/--forever -- run forever (by default, exit when backoff limit is exceeded)
 -h/--help -- print this usage message and exit
 -i/--interactive -- start an interactive shell after executing commands
 -l/--logfile -- log file to be read by logtail command
 -p/--program PROGRAM -- the program to run
--S/--schema XML Schema -- XML schema for configuration file
--T/--start-timeout SECONDS -- Start timeout when a test program is used
 -s/--socket-name SOCKET -- Unix socket name for client (default "zdsock")
 -u/--user USER -- run as this user (or numeric uid)
 -m/--umask UMASK -- use this umask for daemon subprocess (default is 022)
@@ -76,9 +76,9 @@ def string_list(arg):
 
 
 class ZDCtlOptions(RunnerOptions):
-
+    
     __doc__ = __doc__
-
+    
     positional_args_allowed = True
 
     def __init__(self):
@@ -87,16 +87,12 @@ class ZDCtlOptions(RunnerOptions):
                  default="schema.xml",
                  handler=self.set_schemafile)
         self.add("interactive", None, "i", "interactive", flag=1)
-        self.add("default_to_interactive", "runner.default_to_interactive",
-                 default=1)
         self.add("default_to_interactive", "runner.default_to_interactive",
                  default=1)
         self.add("program", "runner.program", "p:", "program=",
                  handler=string_list,
                  required="no program specified; use -p or -C")
         self.add("logfile", "runner.logfile", "l:", "logfile=")
-        self.add("start_timeout", "runner.start_timeout",
-                 "T:", "start-timeout=", int, default=300)
         self.add("python", "runner.python")
         self.add("zdrun", "runner.zdrun")
         programname = os.path.basename(sys.argv[0])
@@ -106,7 +102,7 @@ class ZDCtlOptions(RunnerOptions):
         self.add("prompt", "runner.prompt", default=(programname + ">"))
 
     def realize(self, *args, **kwds):
-
+        
         RunnerOptions.realize(self, *args, **kwds)
 
         # Maybe the config file requires -i or positional args
@@ -183,6 +179,7 @@ class ZDCmd(cmd.Cmd):
         if uid != 0 and uid != self.options.uid:
             self.options.usage("only root can use -u USER to change users")
         os.setgid(self.options.gid)
+        os.setgroups(self.options.groups)
         os.setuid(self.options.uid)
 
     def emptyline(self):
@@ -211,7 +208,6 @@ class ZDCmd(cmd.Cmd):
         except socket.error, msg:
             return None
 
-    zd_testing = 0
     def get_status(self):
         self.zd_up = 0
         self.zd_pid = 0
@@ -225,12 +221,6 @@ class ZDCmd(cmd.Cmd):
         self.zd_up = 1
         self.zd_pid = int(m.group(1))
         self.zd_status = resp
-        m = re.search("(?m)^testing=(\d+)$", resp)
-        if m:
-            self.zd_testing = int(m.group(1))
-        else:
-            self.zd_testing = 0
-
         return resp
 
     def awhile(self, cond, msg):
@@ -240,14 +230,14 @@ class ZDCmd(cmd.Cmd):
             if self.get_status():
                 was_running = True
 
-            while not cond(n):
+            while not cond():
                 sys.stdout.write(". ")
                 sys.stdout.flush()
                 time.sleep(1)
                 n += 1
                 if self.get_status():
                     was_running = True
-                elif (was_running or n > 10) and not cond(n):
+                elif (was_running or n > 10) and not cond():
                     print "\ndaemon manager not running"
                     return
 
@@ -267,13 +257,6 @@ class ZDCmd(cmd.Cmd):
     def help_EOF(self):
         print "To quit, type ^D or use the quit command."
 
-
-    def _start_cond(self, n):
-        if (n > self.options.start_timeout):
-            print '\nProgram took too long to start'
-            sys.exit(1)
-        return self.zd_pid and not self.zd_testing
-
     def do_start(self, arg):
         self.get_status()
         if not self.zd_up:
@@ -308,9 +291,8 @@ class ZDCmd(cmd.Cmd):
             print "daemon process already running; pid=%d" % self.zd_pid
             return
         if self.options.daemon:
-            self.awhile(self._start_cond,
-                        "daemon process started, pid=%(zd_pid)d",
-                        )
+            self.awhile(lambda: self.zd_pid,
+                        "daemon process started, pid=%(zd_pid)d")
 
     def _get_override(self, opt, name, svalue=None, flag=0):
         value = getattr(self.options, name)
@@ -351,7 +333,7 @@ class ZDCmd(cmd.Cmd):
             print "daemon process not running"
         else:
             self.send_action("stop")
-            self.awhile(lambda n: not self.zd_pid, "daemon process stopped")
+            self.awhile(lambda: not self.zd_pid, "daemon process stopped")
 
     def do_reopen_transcript(self, arg):
         if not self.zd_up:
@@ -372,7 +354,7 @@ class ZDCmd(cmd.Cmd):
             self.do_start(arg)
         else:
             self.send_action("restart")
-            self.awhile(lambda n: (self.zd_pid != pid) and self._start_cond(n),
+            self.awhile(lambda: self.zd_pid not in (0, pid),
                         "daemon process restarted, pid=%(zd_pid)d")
 
     def help_restart(self):
@@ -404,7 +386,7 @@ class ZDCmd(cmd.Cmd):
         print "              The default signal is SIGTERM."
 
     def do_wait(self, arg):
-        self.awhile(lambda n: not self.zd_pid, "daemon process stopped")
+        self.awhile(lambda: not self.zd_pid, "daemon process stopped")
         self.do_status()
 
     def help_wait(self):
@@ -463,6 +445,7 @@ class ZDCmd(cmd.Cmd):
         print "umask:       ", oct(umask)
         print "directory:   ", repr(self.options.directory)
         print "logfile:     ", repr(self.options.logfile)
+        print "hang_around: ", repr(self.options.hang_around)
 
     def show_python(self):
         print "Python info:"
@@ -589,8 +572,8 @@ class ZDCmd(cmd.Cmd):
             print "daemon manager not running"
         elif not self.zd_pid:
             print "daemon process not running; stopping daemon manager"
-            self.send_action("stop")
-            self.awhile(lambda n: not self.zd_up, "daemon manager stopped")
+            self.send_action("exit")
+            self.awhile(lambda: not self.zd_up, "daemon manager stopped")
         else:
             print "daemon process and daemon manager still running"
         return 1
