@@ -1,6 +1,6 @@
 ##############################################################################
 #
-# Copyright (c) 2003 Zope Foundation and Contributors.
+# Copyright (c) 2003 Zope Corporation and Contributors.
 # All Rights Reserved.
 #
 # This software is subject to the provisions of the Zope Public License,
@@ -355,6 +355,30 @@ class ZDOptions:
 
 
 class RunnerOptions(ZDOptions):
+    """a zdaemon runner.
+
+    Usage: python <script>.py [-C URL][-h] [zdrun-options] [action [arguments]]
+
+    Options:
+    -C/--configure URL -- configuration file or URL
+    -h/--help -- print usage message and exit
+    -b/--backoff-limit SECONDS -- set backoff limit to SECONDS (default 10)
+    -d/--daemon -- run as a proper daemon; fork a subprocess, close files etc.
+    -f/--forever -- run forever (by default, exit when backoff limit is exceeded)
+    -h/--help -- print this usage message and exit
+    -s/--socket-name SOCKET -- Unix socket name for client (default "zdsock")
+    -u/--user USER -- run as this user (or numeric uid)
+    -m/--umask UMASK -- use this umask for daemon subprocess (default is 022)
+    -x/--exit-codes LIST -- list of fatal exit codes (default "0,2")
+    -z/--directory DIRECTORY -- directory to chdir to when using -d (default off)
+    action [arguments] -- see below
+
+    Actions are commands like "start", "stop" and "status".  If -i is
+    specified or no action is specified on the command line, a "shell"
+    interpreting actions typed interactively is started (unless the
+    configuration option default_to_interactive is set to false).  Use the
+    action "help" to find out about available actions.
+    """
 
     uid = gid = None
 
@@ -374,13 +398,14 @@ class RunnerOptions(ZDOptions):
                  default=022)
         self.add("directory", "runner.directory", "z:", "directory=",
                  existing_parent_directory)
+        self.add("hang_around", "runner.hang_around", default=0)
 
     def realize(self, *args, **kwds):
         ZDOptions.realize(self, *args, **kwds)
 
         # Additional checking of user option; set uid and gid
         if self.user is not None:
-            import pwd
+            import pwd, grp
             try:
                 uid = int(self.user)
             except ValueError:
@@ -388,15 +413,16 @@ class RunnerOptions(ZDOptions):
                     pwrec = pwd.getpwnam(self.user)
                 except KeyError:
                     self.usage("username %r not found" % self.user)
-                uid = pwrec[2]
+                uid = pwrec.pw_uid
             else:
                 try:
                     pwrec = pwd.getpwuid(uid)
                 except KeyError:
                     self.usage("uid %r not found" % self.user)
-            gid = pwrec[3]
             self.uid = uid
-            self.gid = gid
+            self.gid = pwrec.pw_gid
+            self.groups = sorted(g.gr_gid for g in grp.getgrall()
+                                 if self.user in g.gr_mem)
 
 
 # ZConfig datatype
